/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3;

import java.io.PrintStream;

import org.xmloperator.lambda3.model.Application;
import org.xmloperator.lambda3.model.InformantReduction;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermWriter;

/**
 * Implementation of InformantReduction. Shows the reduction state. Decides to
 * stop iterations if there are too much.
 */
class ReductionReport implements InformantReduction {
	private final PrintStream trace;
	private final TermWriter termWriter;
	private final String margin;
	private final int maxStep;
	private int stepCount = 0;

	/**
	 * Constructor.
	 * 
	 * @param trace
	 *            an optional PrintStream.
	 * @param maxStep
	 *            a maximum number of step or -1 if not considered.
	 */
	ReductionReport(PrintStream trace, TermWriter termWriter, int maxStep) {
		this.trace = trace;
		this.termWriter = termWriter;
		this.margin = "		";
		this.maxStep = maxStep;
	}

	int getStepCount() {
		return this.stepCount;
	}

	// /////////////////////////////////
	// InformantReduction implementation
	// /////////////////////////////////

	public void inform(char reductionType, Term rootTerm, Application redex) {
		if (rootTerm == null)
			throw new IllegalArgumentException();
		boolean isStart = (reductionType == InformantReduction.CHAR_REDUCTION_START);
		if (isStart)
			this.stepCount = 0;
		else
			this.stepCount++;
		if (this.trace != null) {
			String stepCountText = new Integer(1000 + (this.stepCount % 1000))
					.toString().substring(1);
			this.trace.print(margin);
			this.trace.print(isStart ? "         " : stepCountText + "--"
					+ reductionType + "-> ");
			this.trace.println(this.termWriter.dagText(rootTerm));
			if (redex != null) {
				this.trace.print(margin);
				this.trace.print("         ");
				this.trace.println(this.termWriter
						.dagRedexUnderscore(rootTerm, redex));
			}
		}
		if (this.maxStep >= 0 && this.stepCount > maxStep)
			throw new TooMuchReductionStepException();
	}
}
