/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.exception.term;

import java.io.PrintStream;

import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermWriter;
import org.xmloperator.lambda3.model.Variable;

/**
 * Exception raised about a Term within a root Term.
 */
public class TermException extends RuntimeException {
	private static final long serialVersionUID = -5379901415607163986L;

	final Term term;
	Term rootTerm;

	public TermException(Term rootTerm, Term term) {
		if (term == null)
			throw new IllegalArgumentException();
		this.term = term;
		this.rootTerm = rootTerm;
	}

	public TermException(Term term) {
		this(null, term);
	}

	public void setRootTerm(Term rootTerm) {
		this.rootTerm = rootTerm;
	}

	public void printTrace(PrintStream trace, TermWriter termWriter) {
		trace.println(this.getClass().getName());
		Term term = this.term;
		if (term.isVariable() && ((Variable) term).getAbstraction() != null) {
			trace.println("Bound variable of");
			term = ((Variable) term).getAbstraction();
		}
		trace.println("	" + termWriter.dagText(term));
		if (this.rootTerm != null)
			trace.println("	" + termWriter.dagText(this.rootTerm));
		this.printStackTrace(trace);
	}
}