/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.impl;

import java.util.List;
import java.util.Map;

import org.xmloperator.lambda3.impl.type.TermTypeFactory;
import org.xmloperator.lambda3.model.Constant;
import org.xmloperator.lambda3.model.Function1;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermRecycler;
import org.xmloperator.lambda3.model.Variable;
import org.xmloperator.lambda3.model.type.TermType;
import org.xmloperator.lambda3.model.type.TermTypeFormula;

/**
 * Implementation of a Constant relative to a function with one argument.
 */
class Constant1Impl extends ConstantImpl<Function1> {

	static final Constant1Impl newConstant(Function1 function, TermType termType) {
		return new Constant1Impl(function, termType);
	}

	static final Constant1Impl newConstant(Function1 function,
			TermTypeFormula termTypeFormula) {
		return new Constant1Impl(function, termTypeFormula);
	}

	private final TermType termType;
	private final TermTypeFormula termTypeFormula;

	Constant1Impl(Function1 function, TermType termType) {
		if (function == null || termType == null)
			throw new IllegalArgumentException();
		this.function = function;
		this.termType = termType;
		this.termTypeFormula = null;
	}

	Constant1Impl(Function1 function, TermTypeFormula termTypeFormula) {
		if (function == null || termTypeFormula == null)
			throw new IllegalArgumentException();
		this.function = function;
		this.termType = null;
		this.termTypeFormula = termTypeFormula;
	}

	// ////////////////////////
	// Constant implementation
	// ////////////////////////

	public final Term apply(Term arg, TermRecycler recycler) {
		TermImpl lastArg = (TermImpl) arg;
		lastArg.referenceCount++;
		TermImpl result = (TermImpl) this.function.function(lastArg);
		result.referenceCount++;
		lastArg.referenceCount--;
		if (lastArg.referenceCount == 0)
			recycler.breakTermDeeply(lastArg);
		result.referenceCount--;
		return result;
	}

	public final Term removeLastArg() {
		return null;
	}

	public final Term getArgument(int index) {
		throw new IllegalArgumentException();
	}

	public Constant cloneWithoutArgument() {
		return this;
	}

	//////////////////////
	// Term implementation
	//////////////////////
	
	TermType assignType(List<Variable> boundVariableStack,
			List<TermType> boundVariableTypeStack,
			Map<Term, TermType> fromSharedTermToType) {
		if (this.termType != null)
			return this.termType;
		if (this.termTypeFormula != null) {
			TermType variableTermType = TermTypeFactory.newVariableTermType();
			return this.termTypeFormula.termTypeFormula(variableTermType);
		}
		throw new IllegalStateException();
	}

	// //////////////////////
	// ConstantImpl override
	// //////////////////////

	final void replaceArgument(int argumentIndex, TermImpl newValue) {
		throw new IllegalArgumentException();
	}

	// //////////////////
	// TermImpl override
	// //////////////////

	final void breakIt() {
		/**
		 * This Constant is never broken.
		 */
	}
}
