/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xmloperator.lambda3.exception.term.StillReferencedTermException;
import org.xmloperator.lambda3.model.Abstraction;
import org.xmloperator.lambda3.model.Application;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermRecycler;
import org.xmloperator.lambda3.model.Variable;
import org.xmloperator.lambda3.model.type.TermType;

/**
 * Term implementation.
 */
abstract class TermImpl implements Term {
	static int REUSE_CAPACITY = 16384;
	static boolean ASSERTION = false;
	
	int referenceCount = 0;

	/**
	 * Constructor.
	 */
	TermImpl() {
	}

	/**
	 * Breaks (and recycles) this Term. Its content is not taken into account.
	 * 
	 * @throws StillReferencedTermException
	 *             if this Term is still referenced.
	 */
	abstract void breakIt();

	/**
	 * Returns a Term equivalent to the abstraction of this Term relatively to
	 * the given bound Variable.
	 * 
	 * @param boundVariable
	 *            a bound Variable.
	 * @return a Term equivalent to the abstraction of this Term relatively to
	 *         the given bound Variable.
	 */
	abstract TermImpl abstractionElimination(Variable boundVariable,
			TermRecycler termRecycler);

	/**
	 * Assigns a type to this Term, if possible.
	 * 
	 * @param boundVariableStack
	 *            the stack of bound variables.
	 * @param boundVariableTypeStack
	 *            the stack of the types assigned to the bound variables.
	 * @param fromSharedTermToType
	 *            the types of the shared terms that have already been visited.
	 * @return the type assigned to this Term.
	 * @throws UntypableTermException
	 *             if it is not possible to assign a type to this Term.
	 */
	abstract TermType assignType(List<Variable> boundVariableStack,
			List<TermType> boundVariableTypeStack,
			Map<Term, TermType> fromSharedTermToType);

	// ////////////////////
	// Term implementation
	// ////////////////////

	public boolean isVariable() {
		return false;
	}

	public boolean isConstant() {
		return false;
	}

	public boolean isAbstraction() {
		return false;
	}

	public boolean isApplication() {
		return false;
	}

	public final int getReferenceCount() {
		return this.referenceCount;
	}

	public final Abstraction abstractOn(Variable freeVariable) {
		return AbstractionImpl
				.newAbstraction((VariableImpl) freeVariable, this);
	}

	public final Application applyTo(Term argument) {
		return ApplicationImpl.newApplication(this, (TermImpl) argument);
	}

	public final TermType assignType() {
		return this.assignType(new ArrayList<Variable>(),
				new ArrayList<TermType>(), new HashMap<Term, TermType>());
	}
}
