/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.impl;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Set;

import org.xmloperator.lambda3.model.Abstraction;
import org.xmloperator.lambda3.model.Application;
import org.xmloperator.lambda3.model.Term;
import org.xmloperator.lambda3.model.TermPrinter;
import org.xmloperator.lambda3.model.TermWriter;

/**
 * TermWriter implementation.
 */
public final class TermWriterImpl implements TermWriter {
	final PrintOptions options = new PrintOptions();

	public TermWriterImpl() {
	}

	// //////////////////////////
	// TermWriter implementation
	// //////////////////////////

	public final String indexedText(Term rootTerm, Term underlinedTerm) {
		StringWriter stringWriter = new StringWriter();
		PrintWriter out = new PrintWriter(stringWriter);
		TermPrinterImpl termPrinter = new TermPrinterImpl(out, underlinedTerm);
		termPrinter.options.copy(this.options);
		termPrinter.printAsIndexed(rootTerm);
		return stringWriter.toString();
	}

	public final String indexedText(Term rootTerm) {
		return indexedText(rootTerm, null);
	}

	public final String lambdaText(Term rootTerm, Term underlinedTerm) {
		StringWriter stringWriter = new StringWriter();
		PrintWriter out = new PrintWriter(stringWriter);
		TermPrinterImpl termPrinter = new TermPrinterImpl(out, underlinedTerm);
		termPrinter.options.copy(this.options);
		termPrinter.printAsLambda(rootTerm);
		return stringWriter.toString();
	}

	public final String lambdaText(Term rootTerm) {
		return lambdaText(rootTerm, null);
	}

	public final String dagText(Term rootTerm, Term underlinedTerm) {
		StringWriter stringWriter = new StringWriter();
		PrintWriter out = new PrintWriter(stringWriter);
		TermPrinterImpl termPrinter = new TermPrinterImpl(out, underlinedTerm);
		termPrinter.options.copy(this.options);
		termPrinter.printAsDAG(rootTerm);
		return stringWriter.toString();
	}

	public final String dagText(Term rootTerm) {
		return dagText(rootTerm, null);
	}

	public final String dagRedexUnderscore(Term rootTerm, Application redex) {
		String redexUnderscore = dagText(rootTerm, redex);
		String argumentUnderscore = dagText(rootTerm, redex.getArgument());
		if (redexUnderscore.length() != argumentUnderscore.length())
			throw new IllegalStateException();
		StringBuffer sb = new StringBuffer();
		for (int k = 0; k < redexUnderscore.length(); k++) {
			char ch = redexUnderscore.charAt(k);
			if (ch == TermPrinter.CHAR_DASH
					&& argumentUnderscore.charAt(k) == TermPrinter.CHAR_DASH)
				ch = TermPrinter.CHAR_STAR;
			sb.append(ch);
		}
		return sb.toString();
	}

	public final String dagAbstractionSetUnderscore(Term rootTerm,
			Set<Abstraction> abstractionSet) {
		String abstractionSetUnderscore = "";
		for (Abstraction abstraction : abstractionSet) {
			String underscore = dagText(rootTerm, abstraction);
			if (abstractionSetUnderscore.length() == 0)
				abstractionSetUnderscore = underscore;
			else {
				StringBuffer sb = new StringBuffer();
				for (int k = 0; k < underscore.length(); k++) {
					char ch1 = abstractionSetUnderscore.charAt(k);
					char ch2 = underscore.charAt(k);
					char ch = (ch1 == TermPrinter.CHAR_DASH) ? ch1 : ch2;
					sb.append(ch);
				}
				abstractionSetUnderscore = sb.toString();
			}
		}
		return abstractionSetUnderscore;
	}
}
