/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.model;

/**
 * A Term that behave as a constant combinator. It is a closed Term.
 */
public interface Constant extends Term {

	/**
	 * Returns the combinator name of this Constant.
	 * 
	 * @return the combinator name of this Constant.
	 */
	public String getCombinatorName();

	/**
	 * Returns <tt>true</tt> if this Constant can be applied on the given
	 * argument.
	 * 
	 * @param arg
	 *            an argument.
	 * @return <tt>true</tt> if this Constant can be applied on the given
	 *         argument.
	 */
	public boolean acceptArgument(Term arg);

	/**
	 * Apply an argument to this Constant.
	 * <p>
	 * A Constant that corresponds to a function with several arguments can
	 * store the first applied arguments.
	 * </p>
	 * 
	 * @param arg
	 *            a next argument.
	 * @param recyler
	 *            a TermReclycler, if necessary.
	 * @return the resulting Term, which may be this Term but in another state.
	 * @throws IllegalFunctionArgumentException
	 *             if <tt>arg</tt> is not compatible with the constant function.
	 */
	public Term apply(Term arg, TermRecycler recycler);

	/**
	 * Removes and returns the last stored argument, if any. Returns null
	 * elsewhere.
	 * 
	 * @return the last stored argument, if any, null elsewhere.
	 */
	public Term removeLastArg();

	/**
	 * Returns the number of stored arguments.
	 * 
	 * @return the number of stored arguments.
	 */
	public int getArgumentCount();

	/**
	 * Returns the stored argument of the given index.
	 * 
	 * @param index
	 *            an index from 0 to argumentCount-1.
	 * @return the stored argument of the given index.
	 */
	public Term getArgument(int index);

	/**
	 * Returns a clone of this Constant, without argument (if any).
	 * 
	 * @return a clone of this Constant, without argument (if any).
	 */
	public Constant cloneWithoutArgument();

	/**
	 * Returns <tt>true</tt> if this Constant represents an int.
	 * 
	 * @return <tt>true</tt> if this Constant represents an int.
	 */
	public boolean isInt();

	/**
	 * Returns <tt>true</tt> if this Constant represents a float number.
	 * 
	 * @return <tt>true</tt> if this Constant represents an float number.
	 */
	public boolean isFloat();
}
