/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.model;

/**
 * Number coding method.
 */
public interface NumberCoding {

	/**
	 * Predefined introduction chars.
	 */
	public static final char CHURCH = TermPrinter.CHAR_HASH;
	public static final char SCOTT = TermPrinter.CHAR_AT;
	public static final char BINARY = TermPrinter.CHAR_DOLLAR;
	public static final char INT = TermPrinter.CHAR_UNDERSCORE;
	public static final char FLOAT = TermPrinter.CHAR_LIVRE;

	/**
	 * Returns the introduction char of this coding method.
	 * 
	 * @return the introduction char of this coding method.
	 */
	public char getIntroductionChar();

	/**
	 * Returns the lambda Term that represents the given number in this coding
	 * system.
	 * 
	 * @param isSigned
	 *            is <tt>true</tt> if the number is signed.
	 * @param number
	 *            a number, non negative if not signed.
	 * @return the lambda Term that represents the given number in this coding
	 *         system.
	 */
	public Term codeNumber(boolean isSigned, int number);

	/**
	 * Predefined names of functions.
	 */
	public static final String ISZERO = "isZero".intern();
	public static final String SUCC = "succ".intern();
	public static final String PRED = "pred".intern();
	public static final String DIV_BY_TWO = "divByTwo".intern();
	public static final String ADD = "add".intern();
	public static final String SUB = "sub".intern();
	public static final String MULT = "mult".intern();
	public static final String DIV = "div".intern();
	public static final String MOD = "mod".intern();
	public static final String POWER = "power".intern();
	public static final String SIGNED_ISZERO = "SisZero".intern();
	public static final String SIGNED_SUCC = "Ssucc".intern();
	public static final String SIGNED_PRED = "Spred".intern();
	public static final String SIGNED_PLUS = "Splus".intern();
	public static final String SIGNED_MULT = "Smult".intern();
	public static final String FROM_CHURCH = "fromChurch".intern();
	public static final String TO_CHURCH = "toChurch".intern();

	/**
	 * Returns the lambda Term that represents the function with the given name,
	 * in this coding system. Returns null if this function is not implemented.
	 * @param functionName
	 *            a function name, possibly a predefined one.
	 * 
	 * @return the lambda Term that represents the function with the given name,
	 *         in this coding system. Returns null if this function is not
	 *         implemented.
	 */
	public Term codeFunction(String functionName);
}
