/**
 * Copyright (c) 2013 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda3/).
 */
package org.xmloperator.lambda3.model.type;

/**
 * Type assigned to a Term, a la Curry.
 */
public interface TermType {

	/**
	 * Returns <tt>true</tt> if this TermType is a variable TermType.
	 * 
	 * @return <tt>true</tt> if this TermType is a variable TermType.
	 */
	public boolean isVariableTermType();

	/**
	 * Returns <tt>true</tt> if this TermType is a FunctionTermType,
	 * <tt>false</tt> elsewhere. This value does'nt change.
	 * 
	 * @return <tt>true</tt> if this TermType is a FunctionTermType,
	 *         <tt>false</tt> elsewhere.
	 */
	public boolean isFunctionTermType();

	/**
	 * Returns <tt>true</tt> if this TermType is a ConstantTermType,
	 * <tt>false</tt> elsewhere. This value does'nt change.
	 * 
	 * @return <tt>true</tt> if this TermType is a ConstantTermType,
	 *         <tt>false</tt> elsewhere.
	 */
	public boolean isConstantTermType();

	/**
	 * Returns <tt>true</tt> if this TermType is currently the last in the
	 * equivalence chain, <tt>false</tt> elsewhere. A FunctionTermType is always
	 * terminal.
	 * 
	 * @return <tt>true</tt> if this TermType is currently the last in the
	 *         equivalence chain, <tt>false</tt> elsewhere.
	 */
	public boolean isTerminal();

	/**
	 * Returns the TermType that finally represents this one. May be this
	 * TermType but not null.
	 * 
	 * @return the TermType that finally represents this one. May be this
	 *         TermType but not null.
	 */
	public TermType getTerminalType();

	/**
	 * Set this TermType as equivalent to the given one. This TermType must be
	 * terminal.
	 * 
	 * @param replacement
	 *            a TermType that is equivalent to this one.
	 * @throws UntypableTermException
	 *             if this equivalence leads to a loop such as
	 *             <tt>T1 = T1->T2</tt>.
	 */
	public void setEquivalence(TermType replacement);

	/**
	 * Returns <tt>true</tt> if this TermType use the given TermType.
	 * 
	 * @param termType
	 *            a TermType.
	 * @return <tt>true</tt> if this TermType use the given TermType.
	 */
	public boolean isUsing(TermType termType);

	static final char CHAR_TYPE_NAME = 'T';
	static final char CHAR_OPENED_PARENTHESIS = '(';
	static final char CHAR_CLOSED_PARENTHESIS = ')';
	static final String TEXT_RIGHT_ARROW = "->";

	/**
	 * Returns a String representation of this TermType.
	 * 
	 * @return a String representation of this TermType.
	 */
	public String toString();
}
